import { GraphQLError } from 'graphql';
import { IResolvers } from 'mercurius';

export const ROOM_ADDED = 'ROOM_ADDED';
export const ROOM_UPDATED = 'ROOM_UPDATED';

export const roomResolvers: IResolvers = {
  Query: {
    rooms: async (_, __, { prisma }) => {
      return prisma.room.findMany({
        where: { isPrivate: false },
      });
    },
    room: async (_: any, { id }: { id: string }, { prisma }) => {
      return prisma.room.findUnique({
        where: { id },
      });
    },
  },
  Mutation: {
    createRoom: async (
      _,
      { name, description, isPrivate },
      { prisma, userId, pubsub }
    ) => {
      if (!userId) {
        throw new GraphQLError('You must be logged in to create a room', {
          extensions: {
            code: 'UNAUTHENTICATED',
          },
        });
      }

      const room = await prisma.room.create({
        data: {
          name,
          description,
          isPrivate: isPrivate ?? false,
          owner: {
            connect: { id: userId },
          },
          members: {
            connect: { id: userId },
          },
        },
      });

      pubsub.publish({
        topic: ROOM_ADDED,
        payload: { roomAdded: room },
      });

      return room;
    },
    joinRoom: async (
      _,
      { roomId }: { roomId: string },
      { prisma, userId, pubsub }
    ) => {
      if (!userId) {
        throw new GraphQLError('You must be logged in to join a room', {
          extensions: {
            code: 'UNAUTHENTICATED',
          },
        });
      }

      const room = await prisma.room.findUnique({
        where: { id: roomId },
        include: { members: true },
      });

      if (!room) {
        throw new GraphQLError('Room not found', {
          extensions: {
            code: 'NOT_FOUND',
          },
        });
      }

      if (room.isPrivate) {
        // In a real application, you would check if the user has been invited
        throw new GraphQLError(
          'You cannot join a private room without an invitation',
          {
            extensions: {
              code: 'FORBIDDEN',
            },
          }
        );
      }

      // Check if user is already a member
      const isMember = room.members.some(
        (member: { id: string }) => member.id === userId
      );
      if (isMember) {
        return room;
      }

      const updatedRoom = await prisma.room.update({
        where: { id: roomId },
        data: {
          members: {
            connect: { id: userId },
          },
        },
        include: { members: true },
      });

      // Publish room updated event
      pubsub.publish({
        topic: ROOM_UPDATED,
        payload: { roomUpdated: updatedRoom },
      });

      return updatedRoom;
    },
    leaveRoom: async (
      _,
      { roomId }: { roomId: string },
      { prisma, userId, pubsub }
    ) => {
      if (!userId) {
        throw new GraphQLError('You must be logged in to leave a room', {
          extensions: {
            code: 'UNAUTHENTICATED',
          },
        });
      }

      const room = await prisma.room.findUnique({
        where: { id: roomId },
        include: { members: true },
      });

      if (!room) {
        throw new GraphQLError('Room not found', {
          extensions: {
            code: 'NOT_FOUND',
          },
        });
      }

      // Check if user is a member
      const isMember = room.members.some(
        (member: { id: string }) => member.id === userId
      );
      if (!isMember) {
        throw new GraphQLError('You are not a member of this room', {
          extensions: {
            code: 'FORBIDDEN',
          },
        });
      }

      // If user is the owner, they cannot leave
      if (room.ownerId === userId) {
        throw new GraphQLError('You cannot leave a room you own', {
          extensions: {
            code: 'FORBIDDEN',
          },
        });
      }

      const updatedRoom = await prisma.room.update({
        where: { id: roomId },
        data: {
          members: {
            disconnect: { id: userId },
          },
        },
        include: { members: true },
      });

      // Publish room updated event
      pubsub.publish({
        topic: ROOM_UPDATED,
        payload: { roomUpdated: updatedRoom },
      });

      return true;
    },
  },
  Subscription: {
    roomAdded: {
      subscribe: (_, __, { pubsub }) => pubsub.subscribe(ROOM_ADDED),
    },
    roomUpdated: {
      subscribe: (_, __, { pubsub }) => pubsub.subscribe(ROOM_UPDATED),
    },
  },
  Room: {
    messages: async (parent: any, _, { prisma }) =>
      parent.messages
        ? parent.messages
        : prisma.room
            .findUnique({
              where: { id: parent.id },
            })
            .messages({
              orderBy: { createdAt: 'asc' },
            }),
    members: async (parent, _, { prisma }) =>
      parent.members
        ? parent.members
        : prisma.room
            .findUnique({
              where: { id: parent.id },
            })
            .members(),
    owner: async (parent, _, { prisma }) =>
      parent.owner
        ? parent.owner
        : prisma.room
            .findUnique({
              where: { id: parent.id },
            })
            .owner(),
  },
};
