import { GraphQLError } from 'graphql';
import { IResolvers } from 'mercurius';

// In a real application, you would use bcrypt for password hashing
// import bcrypt from 'bcryptjs';
// import jwt from 'jsonwebtoken';

export const userResolvers: IResolvers = {
  Query: {
    me: async (_, __, { prisma, userId }) => {
      // In a real application, you would get the user from the context
      // which would be set by an authentication middleware
      if (!userId) {
        return null;
      }
      return prisma.user.findUnique({
        where: { id: userId },
      });
    },
    users: async (_, __, { prisma }) => {
      return prisma.user.findMany();
    },
    user: async (_, { id }: { id: string }, { prisma }) => {
      return prisma.user.findUnique({
        where: { id },
      });
    },
  },
  Mutation: {
    register: async (
      _: any,
      {
        email,
        username,
        password,
      }: { email: string; username: string; password: string },
      { prisma }
    ) => {
      // Check if user already exists
      const existingUser = await prisma.user.findFirst({
        where: {
          OR: [{ email }, { username }],
        },
      });

      if (existingUser) {
        throw new GraphQLError('User already exists', {
          extensions: {
            code: 'USER_ALREADY_EXISTS',
          },
        });
      }

      // In a real application, you would hash the password
      // const hashedPassword = await bcrypt.hash(password, 10);

      const user = await prisma.user.create({
        data: {
          email,
          username,
          password, // In a real app: hashedPassword
        },
      });

      // In a real application, you would generate a JWT token
      // const token = jwt.sign({ userId: user.id }, process.env.JWT_SECRET);

      return {
        token: 'dummy-token', // In a real app: token
        user,
      };
    },
    login: async (
      _: any,
      { email, password }: { email: string; password: string },
      { prisma }
    ) => {
      const user = await prisma.user.findUnique({
        where: { email },
      });

      if (!user) {
        throw new GraphQLError('Invalid credentials', {
          extensions: {
            code: 'INVALID_CREDENTIALS',
          },
        });
      }

      // In a real application, you would verify the password
      // const valid = await bcrypt.compare(password, user.password);
      const valid = password === user.password; // This is just for demo purposes

      if (!valid) {
        throw new GraphQLError('Invalid credentials', {
          extensions: {
            code: 'INVALID_CREDENTIALS',
          },
        });
      }

      // In a real application, you would generate a JWT token
      // const token = jwt.sign({ userId: user.id }, process.env.JWT_SECRET);

      return {
        token: 'dummy-token', // In a real app: token
        user,
      };
    },
  },
  User: {
    messages: async (user, _, { prisma }) =>
      prisma.user
        .findUnique({
          where: { id: user.id },
        })
        .messages(),
    rooms: async (user, _, { prisma }) =>
      prisma.room.findMany({
        where: {
          OR: [{ ownerId: user.id }, { members: { some: { id: user.id } } }],
        },
      }),
    ownedRooms: async (user, _, { prisma }) =>
      prisma.user
        .findUnique({
          where: { id: user.id },
        })
        .rooms(),
  },
};
