import { createSignal, Show } from 'solid-js';
import { Provider } from '@urql/solid';
import { client } from './lib/graphql-client';
import { LoginForm } from './components/login-form';
import { RegisterForm } from './components/register-form';
import { RoomList } from './components/room-list';
import { ChatRoom } from './components/chat-room';
import { CreateRoom } from './components/create-room';
import './App.css';

function App() {
  const [isAuthenticated, setIsAuthenticated] = createSignal(false);
  const [userId, setUserId] = createSignal('');
  const [selectedRoomId, setSelectedRoomId] = createSignal<string | null>(null);
  const [showRegister, setShowRegister] = createSignal(false);

  // Function to generate avatar URL
  const getUserAvatarUrl = (userId: string, size: number = 40) =>
    `https://avatars.jusemon.com/${userId}?size=${size}`;

  // Check if user is already authenticated
  const checkAuth = () => {
    const token = localStorage.getItem('token');
    const storedUserId = localStorage.getItem('userId');

    if (token && storedUserId) {
      setIsAuthenticated(true);
      setUserId(storedUserId);
    }
  };

  // Call checkAuth on component mount
  checkAuth();

  const handleLoginSuccess = (_: string, id: string) => {
    setIsAuthenticated(true);
    setUserId(id);
  };

  const handleLogout = () => {
    localStorage.removeItem('token');
    localStorage.removeItem('userId');
    setIsAuthenticated(false);
    setUserId('');
    setSelectedRoomId(null);
  };

  const handleSelectRoom = (roomId: string) => {
    setSelectedRoomId(roomId);
  };

  const handleLeaveRoom = () => {
    setSelectedRoomId(null);
  };

  return (
    <Provider value={client}>
      <div class='app'>
        <header class='app-header'>
          <h1>Unreal Chat</h1>
          {isAuthenticated() && (
            <div class='user-profile'>
              <img
                src={getUserAvatarUrl(userId())}
                alt='User avatar'
                class='user-avatar'
              />
              <button class='logout-button' onClick={handleLogout}>
                Logout
              </button>
            </div>
          )}
        </header>

        <main class='app-main'>
          <Show
            when={isAuthenticated()}
            fallback={
              <div class='auth-container'>
                <div class='auth-tabs'>
                  <button
                    class={!showRegister() ? 'active' : ''}
                    onClick={() => setShowRegister(false)}
                  >
                    Login
                  </button>
                  <button
                    class={showRegister() ? 'active' : ''}
                    onClick={() => setShowRegister(true)}
                  >
                    Register
                  </button>
                </div>

                <Show
                  when={showRegister()}
                  fallback={<LoginForm onLoginSuccess={handleLoginSuccess} />}
                >
                  <RegisterForm onRegisterSuccess={handleLoginSuccess} />
                </Show>
              </div>
            }
          >
            <div class='chat-container'>
              <aside class='sidebar'>
                <CreateRoom onRoomCreated={handleSelectRoom} />
                <RoomList
                  onSelectRoom={handleSelectRoom}
                  selectedRoomId={selectedRoomId() || undefined}
                />
              </aside>

              <main class='chat-main'>
                <Show when={selectedRoomId()}>
                  <ChatRoom
                    roomId={selectedRoomId() || ''}
                    userId={userId() || ''}
                    onLeaveRoom={handleLeaveRoom}
                  />
                </Show>
                <Show when={!selectedRoomId()}>
                  <div class='select-room-message'>
                    Select a room to start chatting or create a new one
                  </div>
                </Show>
              </main>
            </div>
          </Show>
        </main>
      </div>
    </Provider>
  );
}

export default App;
